/*
 *    Octave-Gtk
 *       The GUI bridge from octave.
 *    Copyright (C) Feb 2005 Muthiah Annamalai, Octave-GTK
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details. 
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<iostream>
#include<octave/oct.h>
#include<octave/parse.h>
#include<octave/dynamic-ld.h>
#include<octave/oct-map.h>
#include<octave/oct-stream.h>
#include<octave/parse.h> /* for feval */
#include<vector>
#include<string>



using namespace std;

extern "C"
{
#include<gtk/gtk.h>
#include<string.h>

/* for FORK */
#include<sys/types.h>
#include<unistd.h>
}


/* Why are these functions here ?? */
gboolean octave_gtk_function(void* data)
{
	cout<<"Octave-GTK-Function"<<endl;
}

/* Why are these functions here ?? */
void octave_gtk_signal_function(void)
{
	cout<<"Octave-GTK-Signal-Function"<<endl;
}



DEFUN_DLD( gtk_init , args , , " void gtk_init (int * argc,char * **argv); " )
{
#ifdef DEBUG
std::cout<<" gtk_init "<<std::endl;
#endif /* DEBUG */
gtk_init (0,NULL);
return octave_value();
}

DEFUN_DLD( gtk_init_check , args , , " gboolean gtk_init_check (int * argc,char * **argv); " )
{
long int ret_type=(long int) gtk_init_check (0,NULL);
return octave_value(ret_type);
}

DEFUN_DLD( gtk_toolbar_append_element , args , , " GtkWidget* gtk_toolbar_append_element (GtkToolbar * toolbar,GtkToolbarChildType type,GtkWidget * widget,char * text,char * tooltip_text,char * tooltip_private_text,GtkWidget * icon,GtkSignalFunc callback,gpointer user_data); " )
{
GtkToolbar *   toolbar ;
GtkToolbarChildType   type ;
GtkWidget *   widget ;
char *   text ;
char *   tooltip_text ;
char *   tooltip_private_text ;
GtkWidget *   icon ;
string   callback ; 
gpointer   user_data ;
 if(args.length()!= 9 )
  {
   std::cout<< "eg:  GtkWidget* gtk_toolbar_append_element (GtkToolbar * toolbar,GtkToolbarChildType type,GtkWidget * widget,char * text,char * tooltip_text,char * tooltip_private_text,GtkWidget * icon,GtkSignalFunc callback,gpointer user_data); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" gtk_toolbar_append_element "<<std::endl;
#endif /* DEBUG */
toolbar =( GtkToolbar * )args( 0 ).long_value();
type = (  GtkToolbarChildType  ) args( 1 ).long_value();
widget =( GtkWidget * )args( 2 ).long_value();
text = new  char   [args( 3 ).string_value ().length()];
strcpy( (char *) text ,args( 3 ).string_value().c_str());
tooltip_text = new  char   [args( 4 ).string_value ().length()];
strcpy( (char *) tooltip_text ,args( 4 ).string_value().c_str());
tooltip_private_text = new  char   [args( 5 ).string_value ().length()];
strcpy( (char *) tooltip_private_text ,args( 5 ).string_value().c_str());
icon =( GtkWidget * )args( 6 ).long_value();
string callback_fcn =args( 7 ).string_value();
g_object_set_data(G_OBJECT( toolbar ),"activated",
		  (void *)callback_fcn.c_str());

user_data =( gpointer )args( 8 ).long_value();
long int ret_type= (long int) 
	gtk_toolbar_append_element ( toolbar,type,widget,text,
				     tooltip_text,tooltip_private_text,
				     icon,octave_gtk_signal_function,
				     user_data);
delete text;
delete tooltip_text;
delete tooltip_private_text;
return octave_value(ret_type);
}

DEFUN_DLD( gtk_toolbar_insert_element , args , , " GtkWidget* gtk_toolbar_insert_element (GtkToolbar * toolbar,GtkToolbarChildType type,GtkWidget * widget,char * text,char * tooltip_text,char * tooltip_private_text,GtkWidget * icon,GtkSignalFunc callback,gpointer user_data,gint position); " )
{
GtkToolbar *   toolbar ;
GtkToolbarChildType   type ;
GtkWidget *   widget ;
char *   text ;
char *   tooltip_text ;
char *   tooltip_private_text ;
GtkWidget *   icon ;
string   callback ; 
gpointer   user_data ;
gint   position ;
 if(args.length()!= 10 )
  {
   std::cout<< "eg:  GtkWidget* gtk_toolbar_insert_element (GtkToolbar * toolbar,GtkToolbarChildType type,GtkWidget * widget,char * text,char * tooltip_text,char * tooltip_private_text,GtkWidget * icon,GtkSignalFunc callback,gpointer user_data,gint position); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" gtk_toolbar_insert_element "<<std::endl;
#endif /* DEBUG */
toolbar =( GtkToolbar * )args( 0 ).long_value();
type = (  GtkToolbarChildType  ) args( 1 ).long_value();
widget =( GtkWidget * )args( 2 ).long_value();
text = new  char   [args( 3 ).string_value ().length()];
strcpy( (char *) text ,args( 3 ).string_value().c_str());
tooltip_text = new  char   [args( 4 ).string_value ().length()];
strcpy( (char *) tooltip_text ,args( 4 ).string_value().c_str());
tooltip_private_text = new  char   [args( 5 ).string_value ().length()];
strcpy( (char *) tooltip_private_text ,args( 5 ).string_value().c_str());
icon =( GtkWidget * )args( 6 ).long_value();
string callback_fcn =args( 7 ).string_value();
g_object_set_data(G_OBJECT( toolbar ),"activated",
		  (void *)callback_fcn.c_str());

user_data =( gpointer )args( 8 ).long_value();
position = (  gint  ) args( 9 ).int_value();
long int ret_type= (long int) gtk_toolbar_insert_element ( toolbar,type,widget,text,tooltip_text,tooltip_private_text,icon,octave_gtk_signal_function,user_data,position);
delete text;
delete tooltip_text;
delete tooltip_private_text;
return octave_value(ret_type);
}

static void generic_cb(GtkWidget *w,gpointer data)
{
  char *cb;
  string *cbs;
  long int x;
  

  cb=(char *)g_object_get_data(G_OBJECT(w),"cb");
  if(cb==NULL)
    {
      cout<<"er Please set the callback"<<endl;
      return;
    }

  x=(long int)w;
  cbs=new string(cb);
  octave_value_list ovl(octave_value((unsigned long int)x));
  feval(*cbs,ovl);
  delete cbs;
  return;
}


DEFUN_DLD(g_signal_connect,args,,"g_signal_connect(widget,clicked,\
callback)")
{
  long int x;
  string sig;
  string cb;
  GtkWidget *w;
  gpointer data=NULL;
  

#ifdef DEBUG
std::cout<<" g_signal_connect "<<std::endl;
#endif /* DEBUG */

  if(args.length() < 3)
    {
      std::cout<<"eg: g_signal_connect(widget,clicked,callback)"<<std::endl;
      return octave_value(1);
    }
  
  if (args.length ()==4) data=(gpointer)args(3).int_value ();
  x=args(0).int_value();
  w=(GtkWidget *)x;

  sig=args(1).string_value();
  cb=args(2).string_value();

  g_object_set_data(G_OBJECT(w),
		    "cb",
		    g_strdup(cb.c_str()));

  //    g_signal_connect (G_OBJECT (window), "delete_event",
  //      G_CALLBACK (delete_event), NULL);

  g_signal_connect(G_OBJECT(w),sig.c_str(),
	     G_CALLBACK(generic_cb), data);


  return octave_value((double)x); //success.
}

DEFUN_DLD( gtk_file_selection_get_selections , args , , " gchar** gtk_file_selection_get_selections (GtkFileSelection * filesel); " )
{
GtkFileSelection *   filesel ;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  gchar** gtk_file_selection_get_selections (GtkFileSelection * filesel); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" gtk_file_selection_get_selections "<<std::endl;
#endif /* DEBUG */
filesel =( GtkFileSelection * )args( 0 ).long_value();
string  ret_type;
char **ptr;
 ptr= gtk_file_selection_get_selections ( filesel);
 while (ptr!=NULL)
	 {
		 string x(*ptr);
		 ret_type=ret_type+"\n"+x;
		 ptr++;
	 }
return octave_value(ret_type);
}

DEFUN_DLD( gtk_rc_get_default_files , args , , " gchar** gtk_rc_get_default_files (void); " )
{
#ifdef DEBUG
std::cout<<" gtk_rc_get_default_files "<<std::endl;
#endif /* DEBUG */
string  ret_type(gtk_rc_get_default_files ( )[0]);
return octave_value(ret_type);
}

DEFUN_DLD( gtk_selection_data_get_text , args , , " guchar * gtk_selection_data_get_text (GtkSelectionData * selection_data); " )
{
GtkSelectionData *   selection_data ;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  guchar * gtk_selection_data_get_text (GtkSelectionData * selection_data); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" gtk_selection_data_get_text "<<std::endl;
#endif /* DEBUG */
selection_data =( GtkSelectionData * )args( 0 ).long_value();
string  ret_type( (char *)gtk_selection_data_get_text ( selection_data));
return octave_value(ret_type);
}
#if 0
DEFUN_DLD( gtk_toggle_button_get_state , args , , 
	   " guchar * gtk_toggle_button_get_state(GtkToggleButton* selection_data); ")
{
	GtkToggleButton *toggle;
	int state=0;
	if(args.length()!= 1 ){		
		std::cout<< "eg:  guchar * gtk_selection_data_get_text (GtkSelectionData * selection_data); " <<std::endl;
		return octave_value(-1);
	}

#ifdef DEBUG
 std::cout<<" gtk_toggle_button_get_state"<<std::endl;
#endif /* DEBUG */

 toggle =(GtkToggleButton * )args( 0 ).long_value();
 stat=gtk_toggle_button_get
 return octave_value(state);
}
#endif 

DEFUN_DLD(gtk_file_selection_get_widget,args,,
	  "GtkWidget* gtk_file_selection_get_widget(GtkFileSelection *fs,char *widgetname)")
{
	GtkFileSelection *fs=NULL;
	long int w=0;
	string name;

#ifdef DEBUG
 std::cout<<" gtk_file_selection_get_widget"<<std::endl;
#endif /* DEBUG */


	if(args.length() != 2){
		std::cout<< "eg:  guchar * gtk_file_selection_get_widget(GtkFileSelection *fs,char *widget);"<<std::endl;
		return octave_value(-1);
	}
	fs = (GtkFileSelection *)args(0).long_value();
	name=args(1).string_value();
	
	switch(name[0])
		{
		case 'o':
		case 'O':
			w=(long int)fs->ok_button;
			break;
		case 'c':
		case 'C':				
			w=(long int)fs->cancel_button;
			break;
		case 'h':
		case 'H':
			w=(long int)fs->help_button;
		default:
			w=(long int)fs->ok_button;
		}
	return octave_value(w);
}
DEFUN_DLD(octave_gtk_main,args,,
	  "void octave_gtk_main(void): forks the gtk_main() into a new process, and return to Octave.")
{

#ifdef DEBUG
 std::cout<<" octave_gtk_main"<<std::endl;
#endif /* DEBUG */

 int childpid=0;
 if((childpid=fork())==0)
	 {
		 /* CHild */
		 gtk_main();
		 return octave_value("process over: gtk_main()");
	 }
 /* Parent */
 return octave_value(childpid);
}

static void timeout_cb(char *callback)
{
  string *cbs;
  if(callback==NULL)
    {
      cout<<"err: Please set the callback"<<endl;
      return;
    }

  cbs=new string(callback);
  //  octave_value_list ovl();
  feval(*cbs);//,ovl);
  delete cbs;
  return;
}
extern char *strdup(const char *ptr);

DEFUN_DLD(gtk_timeout_add, args , ," guint gtk_timeout_add (guint32 interval,GtkFunction function,gpointer data); " )
{
  guint32   interval ;
  string   function ; 
  gpointer   data ;
	
  if(args.length()!= 2 )
  {
   std::cout<< "eg:  guint gtk_timeout_add (guint32 interval,GtkFunction function); " <<std::endl;
   return octave_value(-1);
  }

#ifdef DEBUG
 std::cout<<" gtk_timeout_add "<<std::endl;
#endif /* DEBUG */

 interval = (  guint32  ) args( 0 ).int_value();
 string callback_fcn =args( 1 ).string_value();
 long int ret_type=(long int) gtk_timeout_add(interval,(GtkFunction)timeout_cb,
					      static_cast<void *> (strdup(callback_fcn.c_str())));			   
 
 return octave_value(ret_type);
}


#if 0
DEFUN_DLD( gtk_calendar_get_display_options , args , , "GtkCalendarDisplayOptions gtk_calendar_get_display_options (GtkCalendar * calendar); " )
{
GtkCalendar *   calendar ;
long int flags=-1;
 if(args.length()!= 1 )
  {
   std::cout<< "eg:  int gtk_calendar_get_display_options (GtkCalendar * calendar); " <<std::endl;
   return octave_value(-1);
  }
#ifdef DEBUG
std::cout<<" gtk_calendar_get_display_options "<<std::endl;
#endif /* DEBUG */
calendar =( GtkCalendar * )args( 0 ).long_value();
flags =gtk_calendar_get_display_options ( calendar);
return octave_value(flags);
}
#endif 
